@@ -1,5 +1,5 @@
 /*
- *$Id: ud_socket.h,v 1.1 2001/08/16 02:22:18 sunthockin Exp $
+ *$Id: ud_socket.h,v 1.3 2009/04/22 18:22:28 thockin Exp $
  */
 
 #ifndef UD_SOCKET_H__
@@ -10,5 +10,7 @@
 
 int ud_create_socket(const char *name);
 int ud_accept(int sock, struct ucred *cred);
+int ud_connect(const char *name);
+int ud_get_peercred(int fd, struct ucred *cred);
 
 #endif
